"""
Warning: This part of the client is in beta and its public interface may still change.
"""

import logging

import requests

from clarifai.rest.http_client import HttpClient

logger = logging.getLogger('clarifai')

RETRIES = 2  # if connections fail retry a couple times.
CONNECTIONS = 20  # number of connections to maintain in pool.


class ModerationSolution(object):

  def __init__(self, api_key, base_url='https://api.clarifai-moderation.com/v2'):
    self.api_key = api_key
    self.base_url = base_url

    session = self._make_requests_session()
    self.http_client = HttpClient(session, api_key)

  def _make_requests_session(self):
    http_adapter = requests.adapters.HTTPAdapter(
        max_retries=RETRIES, pool_connections=CONNECTIONS, pool_maxsize=CONNECTIONS)

    session = requests.Session()
    session.mount('http://', http_adapter)
    session.mount('https://', http_adapter)
    return session

  def predict_model(self, model_id, url):
    endpoint_url = '%s/models/%s/outputs' % (self.base_url, model_id)

    payload = {'inputs': [{'data': {'image': {'url': url}}}]}

    return self.http_client.execute_request('POST', payload, endpoint_url)

  def get_moderation_status(self, input_id):
    endpoint_url = '%s/inputs/%s/outputs' % (self.base_url, input_id)

    return self.http_client.execute_request('GET', None, endpoint_url)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/clarifai/api/focus.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from clarifai.rest.grpc.proto.clarifai.api.utils import extensions_pb2 as proto_dot_clarifai_dot_api_dot_utils_dot_extensions__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/clarifai/api/focus.proto',
  package='clarifai.api',
  syntax='proto3',
  serialized_pb=_b('\n\x1eproto/clarifai/api/focus.proto\x12\x0c\x63larifai.api\x1a)proto/clarifai/api/utils/extensions.proto\"3\n\x05\x46ocus\x12\x15\n\x07\x64\x65nsity\x18\x01 \x01(\x02\x42\x04\x80\xb5\x18\x01\x12\x13\n\x05value\x18\x02 \x01(\x02\x42\x04\x80\xb5\x18\x01\x42$Z\x03\x61pi\xa2\x02\x04\x43\x41IP\xc2\x02\x01_\xca\x02\x11\x43larifai\\Internalb\x06proto3')
  ,
  dependencies=[proto_dot_clarifai_dot_api_dot_utils_dot_extensions__pb2.DESCRIPTOR,])




_FOCUS = _descriptor.Descriptor(
  name='Focus',
  full_name='clarifai.api.Focus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='density', full_name='clarifai.api.Focus.density', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\200\265\030\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='clarifai.api.Focus.value', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\200\265\030\001')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=91,
  serialized_end=142,
)

DESCRIPTOR.message_types_by_name['Focus'] = _FOCUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Focus = _reflection.GeneratedProtocolMessageType('Focus', (_message.Message,), dict(
  DESCRIPTOR = _FOCUS,
  __module__ = 'proto.clarifai.api.focus_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.Focus)
  ))
_sym_db.RegisterMessage(Focus)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('Z\003api\242\002\004CAIP\302\002\001_\312\002\021Clarifai\\Internal'))
_FOCUS.fields_by_name['density'].has_options = True
_FOCUS.fields_by_name['density']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\200\265\030\001'))
_FOCUS.fields_by_name['value'].has_options = True
_FOCUS.fields_by_name['value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\200\265\030\001'))
# @@protoc_insertion_point(module_scope)



import os


def _input_url(filename):
  return 'https://samples.clarifai.com/' + filename


def _input_file_path(filename):
  return os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data', filename)


METRO_IMAGE_URL = _input_url('metro-north.jpg')
WEDDING_IMAGE_URL = _input_url('wedding.jpg')
FACEBOOK_IMAGE_URL = _input_url('facebook.png')
DOG_TIFF_IMAGE_URL = _input_url('dog.tiff')
DOG2_IMAGE_URL = _input_url('dog2.jpeg')
DOG2_NONEXISTENT_IMAGE_URL = _input_url('dog2_bad.jpeg')
PENGUIN_BMP_IMAGE_URL = _input_url('penguin.bmp')
TODDLER_FLOWERS_IMAGE_URL = _input_url('toddler-flowers.jpeg')

BEER_VIDEO_URL = _input_url('beer.mp4')
CONAN_GIF_VIDEO_URL = _input_url('3o6gb3kkXfLvdKEZs4.gif')
SMALL_GIF_VIDEO_URL = _input_url('D7qTae7IQLKSI.gif')

METRO_IMAGE_FILE_PATH = _input_file_path('metro-north.jpg')
TAHOE_IMAGE_FILE_PATH = _input_file_path('tahoe.jpg')
TODDLER_FLOWERS_IMAGE_FILE_PATH = _input_file_path('toddler-flowers.jpeg')
THAI_MARKET_IMAGE_FILE_PATH = _input_file_path('thai-market.jpg')

SMALL_VIDEO_FILE_PATH = _input_file_path('small.mp4')
BEER_VIDEO_FILE_PATH = _input_file_path('beer.mp4')



class MockValidator(object):

  def __init__(self, validator):
    # validator is a function that takes a single argument and returns a bool.
    self.validator = validator

  def __eq__(self, other):
    return bool(self.validator(other))


def mock_string_should_end_with(str):
  return MockValidator(lambda x: x.endswith(str))



import logging
import unittest

from clarifai.rest import ApiError, ClarifaiApp
from clarifai.rest import Image as ClImage
from clarifai.rest import UserError

from . import sample_inputs

URLS = [
    sample_inputs.METRO_IMAGE_URL,
    sample_inputs.WEDDING_IMAGE_URL,
    sample_inputs.FACEBOOK_IMAGE_URL,
    sample_inputs.DOG_TIFF_IMAGE_URL,
    sample_inputs.DOG2_IMAGE_URL,
    sample_inputs.DOG2_NONEXISTENT_IMAGE_URL,
    sample_inputs.PENGUIN_BMP_IMAGE_URL,
]

FILE_PATHS = [
    sample_inputs.METRO_IMAGE_FILE_PATH,
    sample_inputs.THAI_MARKET_IMAGE_FILE_PATH,
]


# Tagging is deprecated but the tests are kept to ensure backward-compatibility.
class TestV2Tagging(unittest.TestCase):
  """ the unit test for the V2 simplified tagging api
  """

  @classmethod
  def setUpClass(cls):
    cls.app = ClarifaiApp(log_level=logging.WARN)

  def test_tag_urls(self):
    res = self.app.tag_urls(URLS)
    results = res['outputs']
    self.assertEqual(len(results), len(URLS))

  def test_tag_urls_with_model_id(self):
    # general model
    res = self.app.tag_urls(URLS, model_id='aaa03c23b3724a16a56b629203edc62c')
    results = res['outputs']
    self.assertEqual(len(results), len(URLS))

    # travel model
    res = self.app.tag_urls(URLS, model_id='eee28c313d69466f836ab83287a54ed9')
    results = res['outputs']
    self.assertEqual(len(results), len(URLS))

  def test_tag_non_urls(self):
    """ tag non list or non urls """

    with self.assertRaises(UserError) as ue:
      self.app.tag_urls(sample_inputs.TODDLER_FLOWERS_IMAGE_URL)

  def test_tag_overloaded_urls(self):
    """ tag more than 128 urls """

    urls = [sample_inputs.TODDLER_FLOWERS_IMAGE_URL] * 129

    with self.assertRaises(UserError) as ue:
      self.app.tag_urls(urls)

    urls = [sample_inputs.TODDLER_FLOWERS_IMAGE_URL] * 16
    try:
      res = self.app.tag_urls(urls)
    except ApiError as e:
      # ignore the partial errors
      if e.response.status_code == 207:
        pass
      else:
        raise e

    self.assertEqual(len(res['outputs']), len(urls))

  def test_tag_files(self):
    res = self.app.tag_files(FILE_PATHS)
    results = res['outputs']
    self.assertEqual(len(results), len(FILE_PATHS))

  def test_tag_non_files(self):
    """ tag non list or non filename """

    with self.assertRaises(UserError):
      self.app.tag_files(sample_inputs.METRO_IMAGE_FILE_PATH)

  def test_tag_overloaded_files(self):
    """ tag more than 128 files """

    fs = [sample_inputs.METRO_IMAGE_FILE_PATH] * 129

    with self.assertRaises(UserError):
      self.app.tag_files(fs)

    fs = [sample_inputs.METRO_IMAGE_FILE_PATH] * 16
    self.app.tag_files(fs)

  def test_shortcut_predict_concept_function(self):
    img1 = ClImage(url=sample_inputs.METRO_IMAGE_URL)
    res = self.app.api.predict_concepts([img1])
    self.assertEqual(10000, res['status']['code'])

  def test_shortcut_predict_colors_function(self):
    img1 = ClImage(url=sample_inputs.METRO_IMAGE_URL)
    res = self.app.api.predict_colors([img1])
    self.assertEqual(10000, res['status']['code'])

  def test_shortcut_predict_embed_function(self):
    img1 = ClImage(url=sample_inputs.METRO_IMAGE_URL)
    res = self.app.api.predict_embed([img1])
    self.assertEqual(10000, res['status']['code'])



import copy
import json
import logging

from pprint import pformat
import requests

from clarifai.errors import ApiError
from clarifai.versions import CLIENT_VERSION, OS_VER, PYTHON_VERSION

logger = logging.getLogger('clarifai')


class HttpClient:

  def __init__(self, session, api_key):
    self._api_key = api_key
    self._session = session

  def execute_request(self, method, params, url):
    headers = {
        'Content-Type': 'application/json',
        'X-Clarifai-Client': 'python:%s' % CLIENT_VERSION,
        'Python-Client': '%s:%s' % (OS_VER, PYTHON_VERSION),
        'Authorization': 'Key %s' % self._api_key
    }
    logger.debug("=" * 100)
    succinct_payload = self._mangle_base64_values(params)
    logger.debug("%s %s\nHEADERS:\n%s\nPAYLOAD:\n%s", method, url, pformat(headers),
                 pformat(succinct_payload))
    try:
      if method == 'GET':
        res = self._session.get(url, params=params, headers=headers)
      elif method == "POST":
        res = self._session.post(url, data=json.dumps(params), headers=headers)
      elif method == "DELETE":
        res = self._session.delete(url, data=json.dumps(params), headers=headers)
      elif method == "PATCH":
        res = self._session.patch(url, data=json.dumps(params), headers=headers)
      elif method == "PUT":
        res = self._session.put(url, data=json.dumps(params), headers=headers)
      else:
        raise Exception("Unsupported request type: '%s'" % method)
    except requests.RequestException as e:
      raise ApiError(url, params, method, e.response)
    try:
      response_json = json.loads(res.content.decode('utf-8'))
    except ValueError:
      logger.exception("Could not get valid JSON from server response.")
      logger.debug("\nRESULT:\n%s", pformat(res.content.decode('utf-8')))
      error = ApiError(url, params, method, res)
      raise error
    else:
      logger.debug("\nRESULT:\n%s", pformat(response_json))
    if int(res.status_code / 100) != 2:
      error = ApiError(url, params, method, res)
      logger.warn("%s", str(error))
      raise error
    return response_json

  def _mangle_base64_values(self, params):
    """ Mangle (shorten) the base64 values because they are too long for output. """
    inputs = (params or {}).get('inputs')
    query = (params or {}).get('query')
    if inputs and len(inputs) > 0:
      return self._mangle_base64_values_in_inputs(params)
    if query and query.get('ands'):
      return self._mangle_base64_values_in_query(params)
    return params

  def _mangle_base64_values_in_inputs(self, params):
    params_copy = copy.deepcopy(params)
    for data in params_copy['inputs']:
      data = data['data']
      image = data.get('image')
      if image and image.get('base64'):
        image['base64'] = self._shortened_base64_value(image['base64'])

      video = data.get('video')
      if video and video.get('base64'):
        video['base64'] = self._shortened_base64_value(video['base64'])
    return params_copy

  def _mangle_base64_values_in_query(self, params):
    params_copy = copy.deepcopy(params)
    queries = params_copy['query']['ands']
    for query in queries:
      image = query.get('output', {}).get('input', {}).get('data', {}).get('image', {})
      base64_val = image.get('base64')
      if base64_val:
        image['base64'] = self._shortened_base64_value(base64_val)
    return params_copy

  def _shortened_base64_value(self, original_base64):
    # Shorten the value if larger than what we shorten to (10 + 6 + 10).
    if len(original_base64) > 36:
      return original_base64[:10] + '......' + original_base64[-10:]
    else:
      return original_base64



import mock

from clarifai.rest import ClarifaiApp
from .mock_extensions import mock_string_should_end_with


@mock.patch('clarifai.rest.http_client.HttpClient')
def test_get_all_concepts(mock_http_client):
  mock_execute_request = mock_http_client.return_value.execute_request
  mock_execute_request.side_effect = [{
      'app_id': '',
      'models': []
  }, {
      'app_id':
          '',
      'concepts': [
          {
              'id': '1',
              'name': '1',
              'value': 1,
              'app_id': 'app',
              'created_at': '2010-01-01T00:00:00Z',
          },
          {
              'id': '2',
              'name': '2',
              'value': 1,
              'app_id': 'app',
              'created_at': '2010-01-01T00:00:00Z',
          },
      ],
  }, {
      'app_id': '',
      'concepts': [],
  }]

  concepts = list(ClarifaiApp().concepts.get_all())

  assert len(concepts) == 2
  assert mock_execute_request.mock_calls == [
      mock.call('GET', {
          'per_page': 20,
          'page': 1
      }, mock_string_should_end_with(u'/v2/models')),
      mock.call('GET', {
          'per_page': 20,
          'page': 1
      }, mock_string_should_end_with(u'/v2/concepts')),
      mock.call('GET', {
          'per_page': 20,
          'page': 2
      }, mock_string_should_end_with(u'/v2/concepts'))
  ]



"""
Warning: This part of the client is in beta and its public interface may still change.
"""

from clarifai.rest.solutions.moderation import ModerationSolution


class Solutions(object):

  def __init__(self, api_key=None):
    self.moderation = ModerationSolution(api_key)



import base64

import mock

from clarifai.rest import ClarifaiApp
from .mock_extensions import mock_string_should_end_with

TINY_IMAGE_BASE64 = b'R0lGODlhAQABAIABAP///wAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw=='


@mock.patch('clarifai.rest.http_client.HttpClient')
def test_search_by_image_crop_with_url(mock_http_client):
  mock_execute_request = mock_http_client.return_value.execute_request
  mock_execute_request.side_effect = [{
      'app_id': '',
      'models': []
  }, {
      "status": {
          "code": 10000,
          "description": "Ok"
      },
      "id":
          "2844fa805627436c89e93fa8ffb9ecf9",
      "hits": [{
          "score": 1.0,
          "input": {
              "id": "35a63cd065ca4b34bdcadb7b18b93e66",
              "data": {
                  "image": {
                      "url": "https://samples.clarifai.com/dog2.jpeg"
                  }
              },
              "created_at": "2019-01-07T08:49:36.988214Z",
              "modified_at": "2019-01-07T08:49:38.302670Z",
              "status": {
                  "code": 30000,
                  "description": "Download complete"
              }
          }
      }]
  }]

  images = ClarifaiApp().inputs.search_by_image(
      url='https://samples.clarifai.com/puppy.jpeg', crop=[0.0, 0.0, 1.0, 1.0])

  assert len(images) == 1
  assert images[0].score == 1.0

  assert mock_execute_request.mock_calls == [
      mock.call('GET', {
          'per_page': 20,
          'page': 1
      }, mock_string_should_end_with(u'/v2/models')),
      mock.call(
          'POST', {
              'query': {
                  'ands': [{
                      'output': {
                          'input': {
                              'data': {
                                  'image': {
                                      'url': u'https://samples.clarifai.com/puppy.jpeg',
                                      'crop': [0.0, 0.0, 1.0, 1.0]
                                  }
                              }
                          }
                      }
                  }]
              },
              'pagination': {
                  'per_page': 20,
                  'page': 1
              }
          }, mock_string_should_end_with(u'/v2/searches')),
  ]


@mock.patch('clarifai.rest.http_client.HttpClient')
def test_search_by_image_crop_with_base64(mock_http_client):
  mock_execute_request = mock_http_client.return_value.execute_request
  mock_execute_request.side_effect = [{
      'app_id': '',
      'models': []
  }, {
      "status": {
          "code": 10000,
          "description": "Ok"
      },
      "id":
          "2844fa805627436c89e93fa8ffb9ecf9",
      "hits": [{
          "score": 1.0,
          "input": {
              "id": "35a63cd065ca4b34bdcadb7b18b93e66",
              "data": {
                  "image": {
                      "url": "https://samples.clarifai.com/dog2.jpeg"
                  }
              },
              "created_at": "2019-01-07T08:49:36.988214Z",
              "modified_at": "2019-01-07T08:49:38.302670Z",
              "status": {
                  "code": 30000,
                  "description": "Download complete"
              }
          }
      }]
  }]

  images = ClarifaiApp().inputs.search_by_image(
      base64bytes=TINY_IMAGE_BASE64, crop=[0.0, 0.0, 1.0, 1.0])

  assert len(images) == 1
  assert images[0].score == 1.0

  assert mock_execute_request.mock_calls == [
      mock.call('GET', {
          'per_page': 20,
          'page': 1
      }, mock_string_should_end_with(u'/v2/models')),
      mock.call(
          'POST', {
              'query': {
                  'ands': [{
                      'output': {
                          'input': {
                              'data': {
                                  'image': {
                                      'base64': TINY_IMAGE_BASE64.decode(),
                                      'crop': [0.0, 0.0, 1.0, 1.0]
                                  }
                              }
                          }
                      }
                  }]
              },
              'pagination': {
                  'per_page': 20,
                  'page': 1
              }
          }, mock_string_should_end_with(u'/v2/searches')),
  ]


@mock.patch('clarifai.rest.http_client.HttpClient')
def test_search_by_image_crop_with_filename(mock_http_client):
  mock_execute_request = mock_http_client.return_value.execute_request
  mock_execute_request.side_effect = [{
      'app_id': '',
      'models': []
  }, {
      "status": {
          "code": 10000,
          "description": "Ok"
      },
      "id":
          "2844fa805627436c89e93fa8ffb9ecf9",
      "hits": [{
          "score": 1.0,
          "input": {
              "id": "35a63cd065ca4b34bdcadb7b18b93e66",
              "data": {
                  "image": {
                      "url": "https://samples.clarifai.com/dog2.jpeg"
                  }
              },
              "created_at": "2019-01-07T08:49:36.988214Z",
              "modified_at": "2019-01-07T08:49:38.302670Z",
              "status": {
                  "code": 30000,
                  "description": "Download complete"
              }
          }
      }]
  }]

  images = ClarifaiApp().inputs.search_by_image(
      filename='tests/rest_tests/data/tiny_image.gif', crop=[0.0, 0.0, 1.0, 1.0])

  assert len(images) == 1
  assert images[0].score == 1.0

  assert mock_execute_request.mock_calls == [
      mock.call('GET', {
          'per_page': 20,
          'page': 1
      }, mock_string_should_end_with(u'/v2/models')),
      mock.call(
          'POST', {
              'query': {
                  'ands': [{
                      'output': {
                          'input': {
                              'data': {
                                  'image': {
                                      'base64': TINY_IMAGE_BASE64.decode(),
                                      'crop': [0.0, 0.0, 1.0, 1.0]
                                  }
                              }
                          }
                      }
                  }]
              },
              'pagination': {
                  'per_page': 20,
                  'page': 1
              }
          }, mock_string_should_end_with(u'/v2/searches')),
  ]


@mock.patch('clarifai.rest.http_client.HttpClient')
def test_search_by_image_crop_with_bytes(mock_http_client):
  mock_execute_request = mock_http_client.return_value.execute_request
  mock_execute_request.side_effect = [{
      'app_id': '',
      'models': []
  }, {
      "status": {
          "code": 10000,
          "description": "Ok"
      },
      "id":
          "2844fa805627436c89e93fa8ffb9ecf9",
      "hits": [{
          "score": 1.0,
          "input": {
              "id": "35a63cd065ca4b34bdcadb7b18b93e66",
              "data": {
                  "image": {
                      "url": "https://samples.clarifai.com/dog2.jpeg"
                  }
              },
              "created_at": "2019-01-07T08:49:36.988214Z",
              "modified_at": "2019-01-07T08:49:38.302670Z",
              "status": {
                  "code": 30000,
                  "description": "Download complete"
              }
          }
      }]
  }]

  images = ClarifaiApp().inputs.search_by_image(
      imgbytes=base64.b64decode(TINY_IMAGE_BASE64), crop=[0.0, 0.0, 1.0, 1.0])

  assert len(images) == 1
  assert images[0].score == 1.0

  assert mock_execute_request.mock_calls == [
      mock.call('GET', {
          'per_page': 20,
          'page': 1
      }, mock_string_should_end_with(u'/v2/models')),
      mock.call(
          'POST', {
              'query': {
                  'ands': [{
                      'output': {
                          'input': {
                              'data': {
                                  'image': {
                                      'base64': TINY_IMAGE_BASE64.decode(),
                                      'crop': [0.0, 0.0, 1.0, 1.0]
                                  }
                              }
                          }
                      }
                  }]
              },
              'pagination': {
                  'per_page': 20,
                  'page': 1
              }
          }, mock_string_should_end_with(u'/v2/searches')),
  ]



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/clarifai/api/color.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from clarifai.rest.grpc.proto.clarifai.api.utils import extensions_pb2 as proto_dot_clarifai_dot_api_dot_utils_dot_extensions__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/clarifai/api/color.proto',
  package='clarifai.api',
  syntax='proto3',
  serialized_pb=_b('\n\x1eproto/clarifai/api/color.proto\x12\x0c\x63larifai.api\x1a)proto/clarifai/api/utils/extensions.proto\"M\n\x05\x43olor\x12\x0f\n\x07raw_hex\x18\x01 \x01(\t\x12\x1e\n\x03w3c\x18\x02 \x01(\x0b\x32\x11.clarifai.api.W3C\x12\x13\n\x05value\x18\x03 \x01(\x02\x42\x04\x80\xb5\x18\x01\" \n\x03W3C\x12\x0b\n\x03hex\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\tB$Z\x03\x61pi\xa2\x02\x04\x43\x41IP\xc2\x02\x01_\xca\x02\x11\x43larifai\\Internalb\x06proto3')
  ,
  dependencies=[proto_dot_clarifai_dot_api_dot_utils_dot_extensions__pb2.DESCRIPTOR,])




_COLOR = _descriptor.Descriptor(
  name='Color',
  full_name='clarifai.api.Color',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='raw_hex', full_name='clarifai.api.Color.raw_hex', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='w3c', full_name='clarifai.api.Color.w3c', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='clarifai.api.Color.value', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\200\265\030\001')), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=91,
  serialized_end=168,
)


_W3C = _descriptor.Descriptor(
  name='W3C',
  full_name='clarifai.api.W3C',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hex', full_name='clarifai.api.W3C.hex', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='clarifai.api.W3C.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=170,
  serialized_end=202,
)

_COLOR.fields_by_name['w3c'].message_type = _W3C
DESCRIPTOR.message_types_by_name['Color'] = _COLOR
DESCRIPTOR.message_types_by_name['W3C'] = _W3C
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Color = _reflection.GeneratedProtocolMessageType('Color', (_message.Message,), dict(
  DESCRIPTOR = _COLOR,
  __module__ = 'proto.clarifai.api.color_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.Color)
  ))
_sym_db.RegisterMessage(Color)

W3C = _reflection.GeneratedProtocolMessageType('W3C', (_message.Message,), dict(
  DESCRIPTOR = _W3C,
  __module__ = 'proto.clarifai.api.color_pb2'
  # @@protoc_insertion_point(class_scope:clarifai.api.W3C)
  ))
_sym_db.RegisterMessage(W3C)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('Z\003api\242\002\004CAIP\302\002\001_\312\002\021Clarifai\\Internal'))
_COLOR.fields_by_name['value'].has_options = True
_COLOR.fields_by_name['value']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\200\265\030\001'))
# @@protoc_insertion_point(module_scope)



import unittest

from clarifai.rest.grpc.grpc_json_channel import dict_to_protobuf
from clarifai.rest.grpc.proto.clarifai.api.concept_pb2 import Concept as ConceptPB


class TestJsonToProtobuf(unittest.TestCase):

  def test_concept_with_no_value(self):
    converted = dict_to_protobuf(ConceptPB, {'id': 'some-id', 'name': 'Some Name'})
    assert converted.value == 1.0

  def test_concept_with_value_non_zero(self):
    converted = dict_to_protobuf(ConceptPB, {'id': 'some-id', 'name': 'Some Name', 'value': 0.5})
    assert converted.value == 0.5

  def test_concept_with_value_zero(self):
    converted = dict_to_protobuf(ConceptPB, {'id': 'some-id', 'name': 'Some Name', 'value': 0.0})
    assert converted.value == 0.0

  def test_concept_with_value_one(self):
    converted = dict_to_protobuf(ConceptPB, {'id': 'some-id', 'name': 'Some Name', 'value': 1.0})
    assert converted.value == 1.0

  def test_concept_with_new_field(self):
    converted = dict_to_protobuf(
        ConceptPB, {
            'id': 'some-id',
            'name': 'Some Name',
            'value': 1.0,
            'new_field': 'new_value'
        },
        ignore_unknown_fields=True)
    assert not hasattr(converted, 'new_field')
